/*******************************************************************************
* File Name          	: SE200.c
* Author             	: ELV AG
* Version            	: V1.0
* Date               	: 2009-07-18
* Description        	: Game routines for the SE200
********************************************************************************
********************************************************************************
* Processor						: ATmega168 @ 8 MHz
* Compiler						: IAR (V4.30D)
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "includes.h"

/* Private variables ---------------------------------------------------------*/
unsigned char ucButtonpressLong[4]={0,0,0,0};
unsigned char ucButtonpressShort[4]={0,0,0,0};

static unsigned char ucMenuPointer = CHOOSE_GAMEMODE;
static unsigned char ucSeqPos = 0x00;
static unsigned char ucSeqPosAct = 0x00;
static unsigned char ucRoundsToWin = 0x00;
static unsigned char ucGameSequenz[MAX_GAME_STEPS];

/*******************************************************************************
* Function Name  : StartSiren
* Description    : Let the siren sound 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/

void StartSiren (void)
{
	OCR0A = (((MCU_CLOCK/ 8) * 500) / 1000000);   // set timer-value for siren frequency
	TCCR0A = (0<<COM0A1)|(1<<COM0A0)|(0<<COM0B1)|(0<<COM0B0)|(1<<WGM01)|(0<<WGM00);
  TCCR0B = (0<<FOC0A)|(1<<FOC0B)|(0<<WGM02)|(0<<CS02)|(1<<CS01)|(0<<CS00);
}

/*******************************************************************************
* Function Name  : StartSirenValue
* Description    : Let the siren sound with the given frequency
* Input          : -ucValue: frequency
* Output         : None
* Return         : None
*******************************************************************************/

void StartSirenValue (unsigned char ucValue)
{
	OCR0A = ucValue;   // set timer-value for siren frequency
	TCCR0A = (0<<COM0A1)|(1<<COM0A0)|(0<<COM0B1)|(0<<COM0B0)|(1<<WGM01)|(0<<WGM00);
  TCCR0B = (0<<FOC0A)|(1<<FOC0B)|(0<<WGM02)|(0<<CS02)|(1<<CS01)|(0<<CS00);
}

/*******************************************************************************
* Function Name  : StopSiren
* Description    : Stop the siren sound 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void StopSiren (void)
{
	// configure timer/counter 0
	TCCR0B &= ~(1<<CS01);                                 // disable timer/counter 0
	PORTD |= (1<<PD6);
}

/*******************************************************************************
* Function Name  : ShortSiren
* Description    : Plays a short siren signal 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void ShortSiren(void)
{
  StartSiren();
	WAITms(200);
	StopSiren();
  WAITms(600);
}

/*******************************************************************************
* Function Name  : LongSiren
* Description    : Plays a long siren signal 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LongSiren(void)
{
  StartSiren();
	WAITms(800);
	StopSiren();
	WAITms(600);
}

/*******************************************************************************
* Function Name  : ControlButtonLed
* Description    : Control the LED of the buttons
* Input          : -ucButton : Pointer on the button array 
* Output         : None
* Return         : None
*******************************************************************************/

void ControlButtonLed(unsigned char* ucButton)
{	
	unsigned char ucCounter;
  unsigned char ucPortValue = 0x00;
	
  if(ucMenuPointer == CHOOSE_GAMEMODE)
  {
    ucPortValue = (1<<PD0) | (1<<PD1) | (1<<PD2) | (1<<PD3);
  }
  else
  { 
    for(ucCounter = 0;ucCounter < 4;ucCounter++)
    {
      if(* ucButton)
      {
        ucPortValue |= (1 << ucCounter);
      }
      ucButton++;
    }
  }
  
  cli();
    PORTD = ucPortValue;
  sei();
}

/*******************************************************************************
* Function Name  : ControlButtonLed
* Description    : Control the LED of the buttons
* Input          : -ucCommand : ON or OFF 
* Output         : None
* Return         : None
*******************************************************************************/

void ControlAllButtonLedOn(bool ucCommand)
{	
  unsigned char ucPortValue = 0x00;
	
  if(ucCommand == true)
  {
    ucPortValue = (1<<PD0) | (1<<PD1) | (1<<PD2) | (1<<PD3);
  }
  else
  { 
    ucPortValue = 0x00;
  }
  
  cli();
    PORTD = ucPortValue;
  sei();
}

/*******************************************************************************
* Function Name  : ControlButtonSound
* Description    : Plays the sound of the pressed buttons
* Input          : -ucButton : Pointer on the button array 
* Output         : None
* Return         : None
*******************************************************************************/

void ControlButtonSound(unsigned char* ucButton)
{	
	unsigned char ucCounter;
	
  for(ucCounter = 0;ucCounter < 4;ucCounter++)
	{
		if(*ucButton)
		{
      switch(ucCounter)
      {
        case 0x00:
          StartSirenValue(200);
          break;
        
        case 0x01:
          StartSirenValue(210);
          break;
        
        case 0x02:
          StartSirenValue(220);
          break;
        
        case 0x03:
          StartSirenValue(254);
          break;
          
      }
      WAITms(200);
      StopSiren();
      WAITms(300);
		}
		ucButton++;
	}
}

/*******************************************************************************
* Function Name  : ControlButtonSoundSingle
* Description    : Plays the sound of a single button
* Input          : -ucButton : Pointer on the button array 
* Output         : None
* Return         : None
*******************************************************************************/

void ControlButtonSoundSingle(unsigned char* ucButton)
{
  switch(*ucButton)
  {
    case 0x00:
      StartSirenValue(200);
      break;
    
    case 0x01:
      StartSirenValue(210);
      break;
    
    case 0x02:
      StartSirenValue(220);
      break;
    
    case 0x03:
      StartSirenValue(254);
      break;
      
  }
  WAITms(200);
  StopSiren();
  WAITms(300);
}

/*******************************************************************************
* Function Name  : PlaySequenz
* Description    : Play the actual gamesequenz with the LED's and the siren 
* Input          : -ucSequenz : Pointer on the sequence array
									 -ucSeqPos : Value of the actual game sequence position 
* Output         : None
* Return         : None
*******************************************************************************/

void PlaySequenz(unsigned char* ucSequenz, unsigned char ucSeqPos)
{
  unsigned char ucCounter;
  unsigned char ucPortValue = 0x00;

  for(ucCounter=0; ucCounter<ucSeqPos; ucCounter++)
  {
    cli();
      PORTD = 0x00;
    sei();
      
    WAITms(200);
    
    ucPortValue = (1 << * ucSequenz); 
    
    cli();
      PORTD = ucPortValue;
    sei();
      
    ControlButtonSoundSingle(ucSequenz);
    
    cli();
      PORTD = 0x00;
    sei();
      
    WAITms(200);

    ucSequenz++;
  }
}

/*******************************************************************************
* Function Name  : CompButtonSeq
* Description    : Compare of the user interaction and the stored sequence
* Input          : -ucButton : Pointer on the button array 
									 -ucSequenz : Pointer on the sequence array									
									 -ucSeqPos : Value of the actual games equenz position 
									 -ucRoundsToWin : How many rounds have the player to know 
* Output         : The result of the compare 
* Return         : None
*******************************************************************************/

unsigned char CompButtonSeq(unsigned char* ucButton, unsigned char* ucSequenz, unsigned char ucSeqPos, unsigned char ucRoundsToWin)
{
	unsigned char ucCounter;
	
	ucSequenz+=ucSeqPos;
	
	for(ucCounter=0; ucCounter<4; ucCounter++)
	{
		if(* ucButton)
    {
      if(ucCounter == * ucSequenz)	
      {
        if(ucSeqPos==ucRoundsToWin-1)
          return true;
        else
          return conti;
      }
    }
		ucButton++;
	}
	return false;
}


/*******************************************************************************
* Function Name  : InitGameSeq
* Description    : Reset the array where the game sequence is stored 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/

void InitGameSeq(unsigned char* ucSequenz)
{
  unsigned char ucCounter;

	for(ucCounter=0; ucCounter<MAX_GAME_STEPS; ucCounter++)
	{
		*ucSequenz = 0x00;
		ucSequenz++;
	}
}

/*******************************************************************************
* Function Name  : MainMenu
* Description    : Main Menu 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/

void MainMenu(void)
{
  switch(ucMenuPointer)
	{
		case CHOOSE_GAMEMODE: 
			if(ucButtonpressShort[0]){ucRoundsToWin = 0x28; InitGameSeq(ucGameSequenz); ucSeqPos=0x00; ucMenuPointer=ADD_BUTTON;}
			else if(ucButtonpressShort[2]){ucRoundsToWin = 0x14; InitGameSeq(ucGameSequenz); ucSeqPos=0x00; ucMenuPointer=ADD_BUTTON;}
			else if(ucButtonpressShort[1]){ucRoundsToWin = 0x1E; InitGameSeq(ucGameSequenz); ucSeqPos=0x00; ucMenuPointer=ADD_BUTTON;}
			else if(ucButtonpressShort[3]){ucRoundsToWin = 0x0A; InitGameSeq(ucGameSequenz); ucSeqPos=0x00; ucMenuPointer=ADD_BUTTON;}
      break;
	    
		case ADD_BUTTON: 
			AddRandomButton(ucGameSequenz, ucSeqPos);
			ucSeqPos++;
      ucSeqPosAct = 0x00;
			ucMenuPointer=PLAY_SEQUENZ;
			break;
	    
		case PLAY_SEQUENZ:
			PlaySequenz(ucGameSequenz, ucSeqPos);
			ucMenuPointer=GET_SEQUENZ;
			break; 
	    
		case GET_SEQUENZ:
			if(ucNoButton>= 160) 
      {
        ucMenuPointer=PLAY_SOUND_LOOSER;
      }
      else if(ucButtonpressShort[0]||ucButtonpressShort[1]||ucButtonpressShort[2]||ucButtonpressShort[3])
			{
        switch(CompButtonSeq(ucButtonpressShort, ucGameSequenz, ucSeqPosAct, ucRoundsToWin))
				{
          case true:
						ucMenuPointer=PLAY_SOUND_WINNER;
						break;

          case false:
						ucMenuPointer=PLAY_SOUND_LOOSER;
						break;

          case 0x03:
            ucSeqPosAct++;
            if(ucSeqPos == ucSeqPosAct)
            { 
              ucMenuPointer=ADD_BUTTON; 
            }
						break;
				}	
			}
			break;
	      
		case PLAY_SOUND_WINNER:
      ControlAllButtonLedOn(true);
      ShortSiren();
      ControlAllButtonLedOn(false);
			LongSiren();
      ControlAllButtonLedOn(true);
      ShortSiren();
			ucMenuPointer = CHOOSE_GAMEMODE;
			break;
	    
		case PLAY_SOUND_LOOSER:
      ControlAllButtonLedOn(true);
      StartSiren();
      WAITms(1000);
      StopSiren();
      ControlAllButtonLedOn(false);
			ucMenuPointer = CHOOSE_GAMEMODE;
			break;
	}

	ucButtonpressShort[0] = false;
	ucButtonpressShort[1] = false;
	ucButtonpressShort[2] = false;
	ucButtonpressShort[3] = false;
}

/******************* (C) COPYRIGHT 2010 ELV AG *****END OF FILE****/
