/*******************************************************************************
* File Name          	: buttons.c
* Author             	: ELV AG
* Version            	: V1.0
* Date               	: 2009-07-18
* Description        	: Main routines for the SE200
********************************************************************************
********************************************************************************
* Processor						: ATmega168 @ 8 MHz
* Compiler						: IAR (V4.30D)
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "includes.h"

/* Private variables ---------------------------------------------------------*/
unsigned char ucButtonsActive, ucHoldTime, ucNoButton;

/*******************************************************************************
* Function Name  : InitButtons
* Description    : Initialization of the buuton varaibles   
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/

void InitButtons(void)
{ 
  ucButtonsActive = 10;   
  ucHoldTime = 0;      
  ucNoButton = 0;
}

/*******************************************************************************
* Function Name  : GetButton
* Description    : Detects the pressed button and the duration (Short, Medium, Long) 
* Input          : None
* Output         : None
* Return         : The pressed buttons and the duration  
*******************************************************************************/

unsigned char GetButton(void)
{
  static unsigned char ucButton, ucHoldButton = NO_BUTTON;

  ucButton = PINC & 0x1E;

  if (ucButton != NO_BUTTON)  
  {
    ucButtonsActive = 10; 
    if (ucHoldTime == 0xFF) return BUTTON_DOWN; 
    if (ucButton != (ucHoldButton & NO_BUTTON))   
    {
      if (ucHoldTime)      
        ucHoldTime--;      
      else                
        ucHoldButton = ucButton;   
    }
    else                  
    {
      if (ucHoldTime == 15)  
        ucHoldButton |= BUTTON_MEDIUM; 
      if (ucHoldTime < 79)   
        ucHoldTime++;
      else                  
      {
        ucHoldTime = 0xFF;       
        ucHoldButton = NO_BUTTON;      
        return (ucButton | BUTTON_LONG);  
      }
    }
    return BUTTON_DOWN;
  }
	
  if (ucButtonsActive > 1) ucButtonsActive--;     
  ucHoldTime = 0;
  if (ucHoldButton != NO_BUTTON) 
  {
    ucButton = ucHoldButton;     
    ucHoldButton = NO_BUTTON;  
    return (ucButton);
  }
  return NO_BUTTON;
}

/******************* (C) COPYRIGHT 2010 ELV AG *****END OF FILE****/
